
/* Chrysalide - Outil d'analyse de fichiers binaires
 * space.c - définition d'un espace de noms pour les fonctions de scan
 *
 * Copyright (C) 2022 Cyrille Bagard
 *
 *  This file is part of Chrysalide.
 *
 *  Chrysalide is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Chrysalide is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */


#include "space.h"


#include <string.h>


#include "trigger.h"
#include "space-int.h"
#include "../import.h"



/* ------------------------- SOCLE POUR LES ESPACES DE NOMS ------------------------- */


/* Initialise la classe des espaces de noms dynamiques Kaitai. */
static void g_kaitai_namespace_class_init(GKaitaiNamespaceClass *);

/* Initialise une instance d'espace de noms dynamiques Kaitai. */
static void g_kaitai_namespace_init(GKaitaiNamespace *);

/* Supprime toutes les références externes. */
static void g_kaitai_namespace_dispose(GKaitaiNamespace *);

/* Procède à la libération totale de la mémoire. */
static void g_kaitai_namespace_finalize(GKaitaiNamespace *);



/* --------------------- IMPLEMENTATION DES FONCTIONS DE CLASSE --------------------- */


/* Lance une résolution d'élément à solliciter. */
static bool g_kaitai_namespace_resolve(GKaitaiNamespace *, const char *, GScanContext *, GScanScope *, GScanRegisteredItem **);



/* ---------------------------------------------------------------------------------- */
/*                           SOCLE POUR LES ESPACES DE NOMS                           */
/* ---------------------------------------------------------------------------------- */


/* Indique le type défini pour une définition d'espace de noms dynamique Kaitai. */
G_DEFINE_TYPE(GKaitaiNamespace, g_kaitai_namespace, G_TYPE_SCAN_NAMESPACE);


/******************************************************************************
*                                                                             *
*  Paramètres  : klass = classe à initialiser.                                *
*                                                                             *
*  Description : Initialise la classe des espaces de noms dynamiques Kaitai.  *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static void g_kaitai_namespace_class_init(GKaitaiNamespaceClass *klass)
{
    GObjectClass *object;                   /* Autre version de la classe  */
    GScanRegisteredItemClass *registered;   /* Version de classe parente   */

    object = G_OBJECT_CLASS(klass);

    object->dispose = (GObjectFinalizeFunc/* ! */)g_kaitai_namespace_dispose;
    object->finalize = (GObjectFinalizeFunc)g_kaitai_namespace_finalize;

    registered = G_SCAN_REGISTERED_ITEM_CLASS(klass);

    registered->resolve = (resolve_registered_item_fc)g_kaitai_namespace_resolve;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : space = instance à initialiser.                              *
*                                                                             *
*  Description : Initialise une instance d'espace de noms dynamiques Kaitai.  *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static void g_kaitai_namespace_init(GKaitaiNamespace *space)
{

}


/******************************************************************************
*                                                                             *
*  Paramètres  : space = instance d'objet GLib à traiter.                     *
*                                                                             *
*  Description : Supprime toutes les références externes.                     *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static void g_kaitai_namespace_dispose(GKaitaiNamespace *space)
{
    G_OBJECT_CLASS(g_kaitai_namespace_parent_class)->dispose(G_OBJECT(space));

}


/******************************************************************************
*                                                                             *
*  Paramètres  : space = instance d'objet GLib à traiter.                     *
*                                                                             *
*  Description : Procède à la libération totale de la mémoire.                *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static void g_kaitai_namespace_finalize(GKaitaiNamespace *space)
{
    G_OBJECT_CLASS(g_kaitai_namespace_parent_class)->finalize(G_OBJECT(space));

}


/******************************************************************************
*                                                                             *
*  Paramètres  : -                                                            *
*                                                                             *
*  Description : Construit un nouvel espace de noms dynamique pour Kaitai.    *
*                                                                             *
*  Retour      : Adresse de la structure mise en place.                       *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

GScanNamespace *g_kaitai_namespace_new(void)
{
    GScanNamespace *result;                 /* Instance à retourner        */

    result = g_object_new(G_TYPE_KAITAI_NAMESPACE, NULL);

    if (!g_kaitai_namespace_create(G_KAITAI_NAMESPACE(result)))
        g_clear_object(&result);

    return result;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : space = instance d'espace de noms à initialiser.             *
*                name  = désignation du futur espace de noms.                 *
*                                                                             *
*  Description : Met en place un nouvel espace de noms pour scan.             *
*                                                                             *
*  Retour      : Bilan de l'opération.                                        *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

bool g_kaitai_namespace_create(GKaitaiNamespace *space)
{
    bool result;                            /* Bilan à retourner           */

    result = g_scan_namespace_create(G_SCAN_NAMESPACE(space), "kaitai");

    return result;

}



/* ---------------------------------------------------------------------------------- */
/*                       IMPLEMENTATION DES FONCTIONS DE CLASSE                       */
/* ---------------------------------------------------------------------------------- */


/******************************************************************************
*                                                                             *
*  Paramètres  : item   = élément d'appel à consulter.                        *
*                target = désignation de l'objet d'appel à identifier.        *
*                ctx    = contexte de suivi de l'analyse courante.            *
*                scope  = portée courante des variables locales.              *
*                out    = zone d'enregistrement de la résolution opérée. [OUT]*
*                                                                             *
*  Description : Lance une résolution d'élément à solliciter.                 *
*                                                                             *
*  Retour      : Bilan de l'opération : false en cas d'erreur irrécupérable.  *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static bool g_kaitai_namespace_resolve(GKaitaiNamespace *item, const char *target, GScanContext *ctx, GScanScope *scope, GScanRegisteredItem **out)
{
    bool result;                            /* Bilan à retourner           */
    GScanRegisteredItemClass *parent;       /* Version de classe parente   */
    GKaitaiStruct *kstruct;                 /* Lecteur de définition       */

    parent = G_SCAN_REGISTERED_ITEM_CLASS(g_kaitai_namespace_parent_class);

    result = parent->resolve(G_SCAN_REGISTERED_ITEM(item), target, ctx, scope, out);

    if (!result)
    {
        kstruct = load_kaitai_definition(target, NULL);

        if (kstruct != NULL)
        {
            *out = g_kaitai_trigger_new(kstruct);
            result = true;

            g_object_unref(G_OBJECT(kstruct));

        }

    }

    return result;

}
